from unittest import TestCase

from bhr.single_u_borehole import SingleUBorehole


class TestSingleUBorehole(TestCase):
    def setUp(self):
        self.inputs = {
            "borehole_diameter": 0.096,
            "pipe_outer_diameter": 0.032,
            "pipe_dimension_ratio": 18.53,
            # This is not a realistic value,
            # set to make pipe conduction + internal fluid resistance = 0.05 to match tests
            "length": 100,
            "shank_space": 0.016,
            "pipe_conductivity": 0.389,
            "grout_conductivity": 0.6,
            "soil_conductivity": 4.0,
            "fluid_type": "WATER",
        }

    def test_init(self):
        bh = SingleUBorehole(**self.inputs)
        self.assertEqual(bh.length, 100)

    def test_update_beta(self):
        bh = SingleUBorehole(**self.inputs)
        bh.update_beta(mass_flow_rate=0.5, temperature=20)
        self.assertAlmostEqual(bh.pipe_resist, 0.05, delta=0.00001)

    def test_calc_internal_and_grout_resistance(self):
        bh = SingleUBorehole(**self.inputs)
        flow_rate = 0.5
        temperature = 20
        tolerance = 1e-3
        self.assertAlmostEqual(bh.theta_1, 0.33333, delta=tolerance)
        self.assertAlmostEqual(bh.theta_2, 3.0, delta=tolerance)
        self.assertAlmostEqual(bh.calc_total_internal_bh_resistance(flow_rate, temperature), 0.32365, delta=tolerance)
        self.assertAlmostEqual(bh.calc_grout_resistance(flow_rate, temperature), 0.17701, delta=tolerance)

        self.inputs.update({"soil_conductivity": 1.0, "grout_conductivity": 3.6})
        bh = SingleUBorehole(**self.inputs)
        bh.update_beta(mass_flow_rate=0.5, temperature=20)
        self.assertAlmostEqual(bh.calc_total_internal_bh_resistance(flow_rate, temperature), 0.17456, delta=tolerance)
        self.assertAlmostEqual(bh.calc_grout_resistance(flow_rate, temperature), 0.03373, delta=tolerance)

    def test_calc_effective_bh_resistance_uhf(self):
        bh = SingleUBorehole(**self.inputs)
        tolerance = 1e-3
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_uhf(mass_flow_rate=0.5, temperature=20), 0.20435, delta=tolerance
        )

    def test_calc_effective_bh_resistance_ubwt(self):
        bh = SingleUBorehole(**self.inputs)
        tolerance = 1e-3
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_ubwt(mass_flow_rate=0.5, temperature=20), 0.20435, delta=tolerance
        )



from math import log, pi

from bhr.fluid import get_fluid
from bhr.pipe import Pipe
from bhr.utilities import coth, smoothing_function


class Coaxial:
    def __init__(
        self,
        borehole_diameter: float,
        outer_pipe_outer_diameter: float,
        outer_pipe_dimension_ratio: float,
        outer_pipe_conductivity: float,
        inner_pipe_outer_diameter: float,
        inner_pipe_dimension_ratio: float,
        inner_pipe_conductivity: float,
        length: float,
        grout_conductivity: float,
        soil_conductivity: float,
        fluid_type: str,
        fluid_concentration: float,
    ):
        """
        Implementation for computing borehole thermal resistance for a grouted coaxial borehole.

        :param borehole_diameter: borehole diameter, in m.
        :param outer_pipe_outer_diameter: outer diameter of outer pipe, in m.
        :param outer_pipe_dimension_ratio: non-dimensional ratio of outer pipe diameter to thickness.
        :param outer_pipe_conductivity: outer pipe thermal conductivity, in W/m-K.
        :param inner_pipe_outer_diameter: inner diameter of outer pipe, in m.
        :param inner_pipe_dimension_ratio: non-dimensional ratio of inner pipe diameter to thickness.
        :param inner_pipe_conductivity: inner pipe thermal conductivity, in W/m-K.
        :param length: length of borehole from top to bottom, in m.
        :param grout_conductivity: grout thermal conductivity, in W/m-K.
        :param soil_conductivity: pipe thermal conductivity, in W/m-K.
        :param fluid_type: fluid type. "ETHYLALCOHOL", "ETHYLENEGLYCOL", "METHYLALCOHOL",  "PROPYLENEGLYCOL", or "WATER"
        :param fluid_concentration: fractional concentration of antifreeze mixture, from 0-0.6.
        """

        self.borehole_diameter = borehole_diameter
        self.grout_conductivity = grout_conductivity
        self.soil_conductivity = soil_conductivity
        self.fluid = get_fluid(fluid_type, fluid_concentration)
        self.length = length

        self.outer_pipe = Pipe(
            outer_pipe_outer_diameter,
            outer_pipe_dimension_ratio,
            length,
            outer_pipe_conductivity,
            fluid_type,
            fluid_concentration,
        )
        self.inner_pipe = Pipe(
            inner_pipe_outer_diameter,
            inner_pipe_dimension_ratio,
            length,
            inner_pipe_conductivity,
            fluid_type,
            fluid_concentration,
        )

        self.annular_hydraulic_diameter = self.outer_pipe.pipe_inner_diameter - self.inner_pipe.pipe_outer_diameter

    def re_annulus(self, flow_rate, temp):
        """
        Reynolds number for annulus flow

        :param flow_rate: mass flow rate, kg/s
        :param temp: temperature, C
        :return: Reynolds number
        """

        return 4 * flow_rate / (self.fluid.mu(temp) * pi * self.annular_hydraulic_diameter)

    def laminar_nusselt_annulus(self):
        """
        Laminar Nusselt numbers for annulus flow

        Hellström, G. 1991. Ground Heat Storage: Thermal Analyses of Duct Storage Systems.
        Department of Mathematical Physics, University of Lund, Sweden. pp 67-71

        :return nu_ii: Laminar Nusselt number for inner surface of annulus pipe
        :return nu_oo: Laminar Nusselt number for outer annulus pipe surface
        """
        nu_ii = 3.66 + 1.2 * (self.inner_pipe.pipe_outer_diameter / self.outer_pipe.pipe_inner_diameter) ** -0.8
        nu_oo = 3.66 + 1.2 * (self.inner_pipe.pipe_outer_diameter / self.outer_pipe.pipe_inner_diameter) ** 0.5

        return nu_ii, nu_oo

    def turbulent_nusselt_annulus(self, re, temp):
        """
        Turbulent Nusselt numbers for annulus flow

        Grundmann, Rachel Marie. "Improved design methods for ground heat exchangers."
        Master's thesis, Oklahoma State University, 2016.

        Eqns 4.10 and 4.11 based on the Dittus-Boelter equation

        :param re: Reynolds number
        :param temp: temperature, C
        :return nu_ii: Turbulent Nusselt number for inner surface of annulus pipe
        :return nu_oo: Turbulent Nusselt number for outer annulus pipe surface
        """

        pr = self.fluid.prandtl(temp)

        nu_ii = 0.023 * re**0.8 * pr**0.35
        nu_oo = nu_ii

        return nu_ii, nu_oo

    def convective_resist_annulus(self, flow_rate, temp):
        """
        Grundmann, Rachel Marie. "Improved design methods for ground heat exchangers."
        Master's thesis, Oklahoma State University, 2016.

        Eqns 4.4 - 4.11

        :param flow_rate: mass flow rate, kg/s
        :param temp: temperature, C
        :return r_conv_outside_inner_pipe: convective resistances along the outer wall of the inner pipe, K/(W/m)
        :return r_conv_inside_outer_pipe: convective resistance along the inside wall of the outer pipe K/(W/m)
        """

        # limit determined from Hellström, G. 1991. Ground Heat Storage: Thermal Analyses of
        # Duct Storage Systems. Department of Mathematical Physics, University of Lund, Sweden.

        low_reynolds = 2300

        # limit based on Dittus-Boelter equation
        high_reynolds = 10000

        re = self.re_annulus(flow_rate, temp)

        if re < low_reynolds:
            # use this Nusselt number when the flow is laminar
            nu_ii, nu_oo = self.laminar_nusselt_annulus()

        elif low_reynolds <= re < high_reynolds:
            # in between
            nu_ii_low, nu_oo_low = self.laminar_nusselt_annulus()
            nu_ii_high, nu_oo_high = self.turbulent_nusselt_annulus(high_reynolds, temp)
            nu_ii = smoothing_function(re, low_reynolds, high_reynolds, nu_ii_low, nu_ii_high)
            nu_oo = smoothing_function(re, low_reynolds, high_reynolds, nu_oo_low, nu_oo_high)

        else:
            # use this Nusselt number when the flow is fully turbulent
            nu_ii, nu_oo = self.turbulent_nusselt_annulus(re, temp)

        r_conv_outside_inner_pipe = self.annular_hydraulic_diameter / (
            nu_ii * self.fluid.k(temp) * self.inner_pipe.pipe_outer_diameter * pi
        )

        r_conv_inside_outer_pipe = self.annular_hydraulic_diameter / (
            nu_oo * self.fluid.k(temp) * self.outer_pipe.pipe_inner_diameter * pi
        )

        return r_conv_outside_inner_pipe, r_conv_inside_outer_pipe

    def calc_local_bh_resistance(self, flow_rate, temp):
        """
        Grundmann, Rachel Marie. "Improved design methods for ground heat exchangers."
        Master's thesis, Oklahoma State University, 2016.

        Eqns 4.4 and 4.5

        :param flow_rate: mass flow rate, kg/s
        :param temp: temperature, C
        :return local_bh_resist: total local borehole resistance K /(W/m)
        :return r_internal_resist: local internal borehole resistance K /(W/m)
        :return r_borehole_resist: local borehole resistance K /(W/m)

        """
        # resistances progressing from inside to outside
        r_conv_inner_pipe = self.inner_pipe.calc_pipe_internal_conv_resist(flow_rate, temp)
        r_cond_inner_pipe = self.inner_pipe.calc_pipe_cond_resist()
        r_conv_outside_inner_pipe = self.convective_resist_annulus(flow_rate, temp)[0]
        r_conv_inside_outer_pipe = self.convective_resist_annulus(flow_rate, temp)[1]
        r_cond_outer_pipe = self.outer_pipe.calc_pipe_cond_resist()
        r_cond_grout = log(self.borehole_diameter / self.outer_pipe.pipe_outer_diameter) / (
            2 * pi * self.grout_conductivity
        )

        r_internal_resist = sum([r_conv_inner_pipe, r_cond_inner_pipe, r_conv_outside_inner_pipe])
        r_borehole_resist = sum([r_conv_inside_outer_pipe, r_cond_outer_pipe, r_cond_grout])
        local_bh_resist = r_internal_resist + r_borehole_resist

        return [local_bh_resist, r_internal_resist, r_borehole_resist]

    def calc_effective_bh_resistance_uhf(self, flow_rate, temp):
        """
        Grundmann, Rachel Marie. "Improved design methods for ground heat exchangers."
        Master's thesis, Oklahoma State University, 2016.

        Eqn 4.33

        :param flow_rate: mass flow rate, kg/s
        :param temp: temperature, C
        :return effective_bhr_uhf: effective borehole resistance for
                 uniform heat flux boundary condition  [K/(W/m)]
        """

        _, r_a, r_b = self.calc_local_bh_resistance(flow_rate, temp)
        rv = self.length / (flow_rate * self.fluid.cp(temp))  # (K/(w/m)) thermal resistance factor
        effective_bhr_uhf = r_b + 1 / (3 * r_a) * rv**2

        return effective_bhr_uhf

    def calc_effective_bh_resistance_ubwt(self, flow_rate, temp):
        """
        Grundmann, Rachel Marie. "Improved design methods for ground heat exchangers."
        Master's thesis, Oklahoma State University, 2016.

        Eqns 4.28 & 4.29

        :param flow_rate: mass flow rate, kg/s
        :param temp: temperature, C
        :return effective_bhr_ubwt: effective borehole resistance for
                 uniform borehole wall temperature boundary condition  [K/(W/m)]
        """

        _, r_a, r_b = self.calc_local_bh_resistance(flow_rate, temp)
        rv = self.length / (flow_rate * self.fluid.cp(temp))  # (K/(w/m)) thermal resistance factor
        n = rv / (2 * r_b) * (1 + 4 * r_b / r_a) ** (1 / 2)
        effective_bhr_ubwt = r_b * n * coth(n)

        return effective_bhr_ubwt



import logging

from scp.ethyl_alcohol import EthylAlcohol
from scp.ethylene_glycol import EthyleneGlycol
from scp.methyl_alcohol import MethylAlcohol
from scp.propylene_glycol import PropyleneGlycol
from scp.water import Water


def get_fluid(fluid_type: str, fluid_concentration: float = 0):
    if fluid_concentration < 0:
        logging.warning("Attempting to set <0 water-antifreeze mixture concentration.")
        logging.warning("Expect fluid concentration 0 <= x <= 0.6")
        logging.warning("Defaulting to 0.")
        fluid_concentration = 0

    fluid_name = fluid_type.upper()
    if fluid_name == "WATER":
        if fluid_concentration == 0:
            return Water()
        else:
            logging.warning(
                f'Fluid "{fluid_name}" - attempting to set non-zero \
                            water-antifreeze mixture concentration "{fluid_concentration:0.3f}".'
            )
            logging.warning("Defaulting to pure water.")
            return Water()

    if fluid_concentration == 0:
        logging.warning(f'Setting fluid "{fluid_name} with fluid-antifreeze mixture concentration = 0')

    if fluid_name == "ETHYLALCOHOL":
        return EthylAlcohol(fluid_concentration)
    elif fluid_name == "ETHYLENEGLYCOL":
        return EthyleneGlycol(fluid_concentration)
    elif fluid_name == "METHYLALCOHOL":
        return MethylAlcohol(fluid_concentration)
    elif fluid_name == "PROPYLENEGLYCOL":
        return PropyleneGlycol(fluid_concentration)
    else:
        logging.error(f'Unsupported fluid "{fluid_name}"')
        assert False



from unittest import TestCase

from bhr.u_tube import UTube


class TestUTube(TestCase):
    def test_init(self):
        inputs = {
            "pipe_outer_diameter": 0.034,
            "pipe_dimension_ratio": 11,
            "length": 100,
            "shank_space": 0.03,
            "pipe_conductivity": 0.4,
            "fluid_type": "WATER",
        }

        u_tube = UTube(**inputs)

        self.assertEqual(u_tube.length, 100)
        self.assertEqual(u_tube.pipe_length, 200)
        self.assertEqual(u_tube.shank_space, 0.03)



import unittest

from bhr.borehole import Borehole


class TestBorehole(unittest.TestCase):
    def test_init_single_u_uhf(self):
        bh = Borehole()
        bh.init_single_u_borehole(
            borehole_diameter=0.14,
            pipe_outer_diameter=0.042,
            pipe_dimension_ratio=11,
            length=100,
            shank_space=0.01,
            pipe_conductivity=0.4,
            grout_conductivity=1.2,
            soil_conductivity=2.5,
            fluid_type="PROPYLENEGLYCOL",
            fluid_concentration=0.2,
        )

        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.5), 0.20425, delta=0.0001)

    def test_init_single_u_ubwt(self):
        bh = Borehole()
        bh.init_single_u_borehole(
            borehole_diameter=0.14,
            pipe_outer_diameter=0.042,
            pipe_dimension_ratio=11,
            length=100,
            shank_space=0.01,
            pipe_conductivity=0.4,
            grout_conductivity=1.2,
            soil_conductivity=2.5,
            fluid_type="PROPYLENEGLYCOL",
            fluid_concentration=0.2,
            boundary_condition="uniform_borehole_wall_temp",
        )

        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.5), 0.20414, delta=0.0001)

    def test_init_single_u_from_dict(self):
        inputs = {
            "fluid_type": "PROPYLENEGLYCOL",
            "fluid_concentration": 0.2,
            "boundary_condition": "uniform_heat_flux",
            "borehole_type": "single_u_tube",
            "single_u_tube": {
                "pipe_outer_diameter": 0.042,
                "pipe_dimension_ratio": 11,
                "pipe_conductivity": 0.4,
                "shank_space": 0.01,
            },
            "grout_conductivity": 1.2,
            "soil_conductivity": 2.5,
            "length": 100,
            "borehole_diameter": 0.14,
        }

        bh = Borehole()
        bh.init_from_dict(inputs)

        # only pass flow rate, so pipe resistance should be computed in the process of this call

        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.5), 0.20425, delta=0.0001)

    def test_init_double_u_uhf(self):
        bh = Borehole()
        bh.init_double_u_borehole(
            borehole_diameter=0.115,
            pipe_outer_diameter=0.032,
            pipe_dimension_ratio=18.9,
            length=200,
            shank_space=0.02263,
            pipe_conductivity=0.389,
            pipe_inlet_arrangement="ADJACENT",
            grout_conductivity=1.5,
            soil_conductivity=3,
            fluid_type="WATER",
            fluid_concentration=0,
        )

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.4154), 0.1090, delta=0.0001)

    def test_init_double_u_ubwt(self):
        bh = Borehole()
        bh.init_double_u_borehole(
            borehole_diameter=0.115,
            pipe_outer_diameter=0.032,
            pipe_dimension_ratio=18.9,
            length=200,
            shank_space=0.02263,
            pipe_conductivity=0.389,
            pipe_inlet_arrangement="ADJACENT",
            grout_conductivity=1.5,
            soil_conductivity=3,
            fluid_type="WATER",
            fluid_concentration=0,
            boundary_condition="uniform_borehole_wall_temp",
        )

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.4154), 0.1065, delta=0.0001)

    def test_init_double_u_from_dict(self):
        inputs = {
            "fluid_type": "WATER",
            "fluid_concentration": 0,
            "boundary_condition": "uniform_heat_flux",
            "borehole_type": "double_u_tube",
            "double_u_tube": {
                "pipe_outer_diameter": 0.032,
                "pipe_dimension_ratio": 18.9,
                "pipe_conductivity": 0.389,
                "shank_space": 0.02263,
                "pipe_inlet_arrangement": "ADJACENT",  # or DIAGONAL
            },
            "grout_conductivity": 1.5,
            "soil_conductivity": 3,
            "length": 200,
            "borehole_diameter": 0.115,
        }

        bh = Borehole()
        bh.init_from_dict(inputs)

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(temperature=20, mass_flow_rate=0.4154), 0.1090, delta=0.0001)

    def test_init_coaxial_uhf(self):
        bh = Borehole()
        bh.init_coaxial_borehole(
            borehole_diameter=0.115,
            outer_pipe_outer_diameter=0.064,
            outer_pipe_dimension_ratio=11,
            outer_pipe_conductivity=0.389,
            inner_pipe_outer_diameter=0.032,
            inner_pipe_dimension_ratio=11,
            inner_pipe_conductivity=0.389,
            length=200,
            grout_conductivity=1.5,
            soil_conductivity=3.0,
            fluid_type="WATER",
            fluid_concentration=0,
        )

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(mass_flow_rate=0.5, temperature=20), 0.18048, delta=0.0001)

    def test_init_coaxial_ubwt(self):
        bh = Borehole()
        bh.init_coaxial_borehole(
            borehole_diameter=0.115,
            outer_pipe_outer_diameter=0.064,
            outer_pipe_dimension_ratio=11,
            outer_pipe_conductivity=0.389,
            inner_pipe_outer_diameter=0.032,
            inner_pipe_dimension_ratio=11,
            inner_pipe_conductivity=0.389,
            length=200,
            grout_conductivity=1.5,
            soil_conductivity=3.0,
            fluid_type="WATER",
            fluid_concentration=0,
            boundary_condition="uniform_borehole_wall_temp",
        )

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(mass_flow_rate=0.5, temperature=20), 0.18365, delta=0.0001)

    def test_init_coaxial_from_dict(self):
        inputs = {
            "fluid_type": "WATER",
            "fluid_concentration": 0,
            "boundary_condition": "uniform_heat_flux",
            "borehole_type": "coaxial",
            "coaxial": {
                "outer_pipe_outer_diameter": 0.064,
                "outer_pipe_dimension_ratio": 11,
                "outer_pipe_conductivity": 0.389,
                "inner_pipe_outer_diameter": 0.032,
                "inner_pipe_dimension_ratio": 11,
                "inner_pipe_conductivity": 0.389,
            },
            "grout_conductivity": 1.5,
            "soil_conductivity": 3,
            "length": 200,
            "borehole_diameter": 0.115,
        }

        bh = Borehole()
        bh.init_from_dict(inputs)

        # only pass flow rate, so pipe resistance should be computed in the process of this call
        self.assertAlmostEqual(bh.calc_bh_resist(mass_flow_rate=0.5, temperature=20), 0.18048, delta=0.0001)



import unittest

from bhr.fluid import get_fluid


class TestFluid(unittest.TestCase):
    def test_init_ethyl_alcohol(self):
        f = get_fluid(fluid_type="ETHYLALCOHOL", fluid_concentration=0.2)
        self.assertAlmostEqual(f.density(20), 968.9, delta=0.1)

    def test_init_ethylene_glycol(self):
        f = get_fluid(fluid_type="ETHYLENEGLYCOL", fluid_concentration=0.2)
        self.assertAlmostEqual(f.density(20), 1024.1, delta=0.1)

    def test_init_methyl_alcohol(self):
        f = get_fluid(fluid_type="METHYLALCOHOL", fluid_concentration=0.2)
        self.assertAlmostEqual(f.density(20), 966.7, delta=0.1)

    def test_init_propylene_glycol(self):
        f = get_fluid(fluid_type="PROPYLENEGLYCOL", fluid_concentration=0.2)
        self.assertAlmostEqual(f.density(20), 1014.7, delta=0.1)

    def test_init_water(self):
        f = get_fluid(fluid_type="WATER")
        self.assertAlmostEqual(f.density(20), 998.2, delta=0.1)



import unittest
from math import log

from bhr.pipe import Pipe


class TestPipe(unittest.TestCase):
    def setUp(self):
        self.inputs = {
            "pipe_outer_diameter": 0.0334,
            "pipe_dimension_ratio": 11,
            "pipe_length": 100,
            "pipe_conductivity": 0.4,
            "fluid_type": "WATER",
        }

    def test_init_pipe(self):
        p = Pipe(**self.inputs)
        tol = 0.0001

        # props
        self.assertAlmostEqual(p.conductivity, 0.4, delta=tol)

        # geometry
        self.assertAlmostEqual(p.pipe_outer_diameter, 0.0334, delta=tol)
        self.assertAlmostEqual(p.pipe_inner_diameter, 0.0273, delta=tol)
        self.assertAlmostEqual(p.pipe_length, 100, delta=tol)
        self.assertAlmostEqual(p.thickness, 0.00303, delta=tol)

        # areas
        self.assertAlmostEqual(p.area_cr_outer, 8.761e-4, delta=tol)
        self.assertAlmostEqual(p.area_cr_inner, 5.628e-4, delta=tol)
        self.assertAlmostEqual(p.area_cr_pipe, 3.078e-4, delta=tol)
        self.assertAlmostEqual(p.area_s_outer, 10.4929, delta=tol)
        self.assertAlmostEqual(p.area_s_inner, 8.5851, delta=tol)

        # volumes
        self.assertAlmostEqual(p.total_vol, 0.0876, delta=tol)
        self.assertAlmostEqual(p.fluid_vol, 0.0586, delta=tol)
        self.assertAlmostEqual(p.pipe_wall_vol, 0.0289, delta=tol)

    def test_mdot_to_re(self):
        p = Pipe(**self.inputs)
        tol = 0.1
        self.assertAlmostEqual(p.mdot_to_re(0.1, 20), 4649.9, delta=tol)

    def test_calc_friction_factor(self):
        p = Pipe(**self.inputs)
        tol = 1e-4

        # laminar tests
        re = 100
        self.assertEqual(p.friction_factor(re), 64.0 / re)

        re = 1000
        self.assertEqual(p.friction_factor(re), 64.0 / re)

        re = 1400
        self.assertEqual(p.friction_factor(re), 64.0 / re)

        # transitional tests
        re = 2000
        self.assertAlmostEqual(p.friction_factor(re), 0.03213, delta=tol)

        re = 3000
        self.assertAlmostEqual(p.friction_factor(re), 0.03344, delta=tol)

        re = 4000
        self.assertAlmostEqual(p.friction_factor(re), 0.04127, delta=tol)

        # turbulent tests
        re = 5000
        self.assertEqual(p.friction_factor(re), (0.79 * log(re) - 1.64) ** (-2.0))

        re = 15000
        self.assertEqual(p.friction_factor(re), (0.79 * log(re) - 1.64) ** (-2.0))

        re = 25000
        self.assertEqual(p.friction_factor(re), (0.79 * log(re) - 1.64) ** (-2.0))

    def test_laminar_nusselt(self):
        p = Pipe(**self.inputs)
        tol = 0.01
        self.assertAlmostEqual(p.laminar_nusselt(), 4.01, delta=tol)

    def test_turbulent_nusselt(self):
        p = Pipe(**self.inputs)
        tol = 0.01
        self.assertAlmostEqual(p.turbulent_nusselt(3000, 20), 18.39, delta=tol)
        self.assertAlmostEqual(p.turbulent_nusselt(10000, 20), 79.50, delta=tol)

    def test_calc_conduction_resistance(self):
        p = Pipe(**self.inputs)
        tolerance = 0.00001
        self.assertAlmostEqual(p.calc_pipe_cond_resist(), 0.0798443, delta=tolerance)

    def test_calc_convection_resistance(self):
        p = Pipe(**self.inputs)
        temp = 20
        tol = 0.00001
        self.assertAlmostEqual(p.calc_pipe_internal_conv_resist(0, temp), 0.13266, delta=tol)
        self.assertAlmostEqual(p.calc_pipe_internal_conv_resist(0.07, temp), 0.020784, delta=tol)
        self.assertAlmostEqual(p.calc_pipe_internal_conv_resist(2, temp), 0.00094, delta=tol)

    def test_calc_resist(self):
        pipe = Pipe(**self.inputs)
        tol = 0.00001
        self.assertAlmostEqual(pipe.calc_pipe_resist(0.5, 20), 0.082985, delta=tol)

    def test_pressure_loss(self):
        pipe = Pipe(**self.inputs)
        tol = 1
        self.assertAlmostEqual(pipe.pressure_loss(0.5, 20), 33533, delta=tol)



from unittest import TestCase

from bhr.coaxial_borehole import Coaxial


class TestCoaxialBorehole(TestCase):
    def setUp(self):
        self.inputs = {
            "borehole_diameter": 0.115,
            "outer_pipe_outer_diameter": 0.064,
            "outer_pipe_dimension_ratio": 11,
            "outer_pipe_conductivity": 0.389,
            "inner_pipe_outer_diameter": 0.032,
            "inner_pipe_dimension_ratio": 11,
            "inner_pipe_conductivity": 0.389,
            "length": 200,
            "grout_conductivity": 1.5,
            "soil_conductivity": 3,
            "fluid_type": "WATER",
            "fluid_concentration": 0,
        }

    def test_init(self):
        coax = Coaxial(**self.inputs)
        self.assertEqual(coax.borehole_diameter, 0.115)

    def test_re_annulus(self):
        coax = Coaxial(**self.inputs)
        self.assertAlmostEqual(coax.re_annulus(flow_rate=0.5, temp=20), 31200.1777, delta=1e-3)

    def test_laminar_nusselt_annulus(self):
        coax = Coaxial(**self.inputs)

        tolerance = 1e-3

        self.assertAlmostEqual(coax.laminar_nusselt_annulus()[0], 5.4394, delta=tolerance)
        self.assertAlmostEqual(coax.laminar_nusselt_annulus()[1], 4.5980, delta=tolerance)

    def test_turbulent_nusselt_annulus(self):
        coax = Coaxial(**self.inputs)
        self.assertAlmostEqual(coax.turbulent_nusselt_annulus(re=10000, temp=20)[0], 72.0409, delta=1e-3)
        self.assertAlmostEqual(coax.turbulent_nusselt_annulus(re=20000, temp=20)[1], 125.4305, delta=1e-3)

    def test_convective_resist_annulus(self):
        coax = Coaxial(**self.inputs)

        # tests convective resistance of outside surface of inner pipe
        # laminar flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.02, temp=20)[0], 0.062236, delta=1e-3)
        # transitional flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.1, temp=20)[0], 0.00820999, delta=1e-3)
        # turbulent flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.5, temp=20)[0], 0.001891, delta=1e-3)

        # tests convective resistance of inside surface of outer pipe
        # laminar flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.02, temp=20)[1], 0.04499, delta=1e-3)
        # transitional flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.1, temp=20)[1], 0.005065, delta=1e-3)
        # turbulent flow
        self.assertAlmostEqual(coax.convective_resist_annulus(flow_rate=0.5, temp=20)[1], 0.001155, delta=1e-3)

    def test_calc_local_bh_resistance(self):
        coax = Coaxial(**self.inputs)

        # test turbulent flow
        self.assertAlmostEqual(coax.calc_local_bh_resistance(flow_rate=0.5, temp=20)[0], 0.23245, delta=1e-3)
        # test intermediate flow
        self.assertAlmostEqual(coax.calc_local_bh_resistance(flow_rate=0.1, temp=20)[0], 0.2533, delta=1e-3)
        # test laminar flow
        self.assertAlmostEqual(coax.calc_local_bh_resistance(flow_rate=0.02, temp=20)[0], 0.4663, delta=1e-3)

    def test_calc_effective_bh_resistance_uhf(self):
        coax = Coaxial(**self.inputs)

        self.assertAlmostEqual(coax.calc_effective_bh_resistance_uhf(flow_rate=0.02, temp=20), 7.07, delta=1e-3)

    def test_calc_effective_bh_resistance_ubwt(self):
        coax = Coaxial(**self.inputs)

        self.assertAlmostEqual(coax.calc_effective_bh_resistance_ubwt(flow_rate=0.02, temp=20), 2.31, delta=1e-3)



from bhr.pipe import Pipe


class UTube(Pipe):
    def __init__(
        self,
        pipe_outer_diameter: float,
        pipe_dimension_ratio: float,
        length: float,
        shank_space: float,
        pipe_conductivity: float,
        fluid_type: str,
        fluid_concentration: float = 0,
    ):
        super().__init__(
            pipe_outer_diameter, pipe_dimension_ratio, length * 2, pipe_conductivity, fluid_type, fluid_concentration
        )
        self.length = length
        self.shank_space = shank_space



from unittest import TestCase

from bhr.double_u_borehole import DoubleUTube


class TestDoubleUBorehole(TestCase):
    def setUp(self):
        # from example in Claesson & Javed 2019 paper
        self.inputs = {
            "borehole_diameter": 0.115,  # confirmed
            "pipe_outer_diameter": 0.032,  # confirmed
            "pipe_dimension_ratio": 20.164,  # tuned to set Rp = 0.05
            "length": 200,  # confirmed
            "shank_space": 0.02263,  # confirmed
            "pipe_conductivity": 0.389,  # reasonable value, but ultimately effects come out in Rp
            "pipe_inlet_arrangement": "DIAGONAL",  # correct
            "grout_conductivity": 1.5,  # confirmed
            "soil_conductivity": 3,  # confirmed
            "fluid_type": "WATER",  # effects come out in Rp
        }

        self.v_dot_bh = 1.5 / 3600  # m3/s
        self.v_dot_per_u = self.v_dot_bh / 2
        self.rho_test = 997
        self.m_dot_bh = self.v_dot_bh * self.rho_test
        self.m_dot_per_u = self.v_dot_per_u * self.rho_test

    def test_init(self):
        bh = DoubleUTube(**self.inputs)
        self.assertEqual(bh.bh_length, 200)

    def test_shank_space_assert(self):
        # checks if code is raising error when shank space is too large
        d = self.inputs.copy()
        with self.assertRaises(AssertionError):
            d.update({"shank_space": 0.115})
            DoubleUTube(**d)

        # checks if code is raising error when shank space is too small
        d = self.inputs.copy()
        with self.assertRaises(AssertionError):
            d.update({"shank_space": 0.002})
            DoubleUTube(**d)

    def test_update_beta_b1(self):
        bh = DoubleUTube(**self.inputs)

        tolerance = 1e-3

        self.assertAlmostEqual(bh.update_b1(m_dot_per_u_tube=self.m_dot_per_u, temperature=20), 0.359, delta=tolerance)
        self.assertAlmostEqual(bh.pipe_resist, 0.05, delta=tolerance)

    def test_calc_bh_resist_local(self):
        bh = DoubleUTube(**self.inputs)
        tolerance = 1e-3
        self.assertAlmostEqual(
            bh.calc_bh_resist_local(m_dot_per_u_tube=self.m_dot_per_u, temperature=20), 7.509e-02, delta=tolerance
        )

    def test_calc_internal_resist(self):
        bh = DoubleUTube(**self.inputs)
        tolerance = 1e-3
        self.assertAlmostEqual(
            bh.calc_internal_resist(m_dot_per_u_tube=self.m_dot_per_u, temperature=20), 0.1604, delta=tolerance
        )

    def test_calc_resistances_diagonal(self):
        d = self.inputs.copy()
        d.update({"pipe_inlet_arrangement": "DIAGONAL"})
        bh = DoubleUTube(**d)
        tolerance = 1e-3

        # values match test case data in Table 1
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_uhf(mass_flow_rate=self.m_dot_bh, temperature=20), 0.1302, delta=tolerance
        )
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_ubwt(mass_flow_rate=self.m_dot_bh, temperature=20), 0.1235, delta=tolerance
        )

    def test_calc_resistances_adjacent(self):
        d = self.inputs.copy()
        d.update({"pipe_inlet_arrangement": "ADJACENT"})
        bh = DoubleUTube(**d)
        tolerance = 1e-3

        # values match test case data in Table 1
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_uhf(mass_flow_rate=self.m_dot_bh, temperature=20), 0.1089, delta=tolerance
        )
        self.assertAlmostEqual(
            bh.calc_effective_bh_resistance_ubwt(mass_flow_rate=self.m_dot_bh, temperature=20), 0.1062, delta=tolerance
        )

    def test_invalid_pipe_arrangement(self):
        d = self.inputs.copy()
        d.update({"pipe_inlet_arrangement": "UNSUPPORTED"})
        with self.assertRaises(AssertionError):
            DoubleUTube(**d)



from bhr.coaxial_borehole import Coaxial
from bhr.double_u_borehole import DoubleUTube
from bhr.enums import BoreholeType, BoundaryCondition
from bhr.single_u_borehole import SingleUBorehole
from bhr.utilities import set_boundary_condition_enum


class Borehole:
    def __init__(self):
        self._bh_type = None
        self._boundary_condition = None
        self._bh = None

    def init_single_u_borehole(
        self,
        borehole_diameter: float,
        pipe_outer_diameter: float,
        pipe_dimension_ratio: float,
        length: float,
        shank_space: float,
        pipe_conductivity: float,
        grout_conductivity: float,
        soil_conductivity: float,
        fluid_type: str,
        fluid_concentration: float = 0,
        boundary_condition: str = "UNIFORM_HEAT_FLUX",
    ):
        """
        Constructs a grouted single u-tube borehole.

        :param borehole_diameter: borehole diameter, in m.
        :param pipe_outer_diameter: outer diameter of the pipe, in m.
        :param pipe_dimension_ratio: non-dimensional ratio of pipe diameter to pipe thickness.
        :param length: length of borehole from top to bottom, in m.
        :param shank_space: radial distance from the borehole center to the pipe center, in m.
        :param pipe_conductivity: pipe thermal conductivity, in W/m-K.
        :param grout_conductivity: grout thermal conductivity, in W/m-K.
        :param soil_conductivity: soil thermal conductivity, in W/m-K.
        :param fluid_type: fluid type. "ETHYLALCOHOL", "ETHYLENEGLYCOL", "METHYLALCOHOL",  "PROPYLENEGLYCOL", or "WATER"
        :param fluid_concentration: fractional concentration of antifreeze mixture, from 0-0.6.
        :param boundary_condition: borehole wall boundary condition. "UNIFORM_HEAT_FLUX" or "UNIFORM_BOREHOLE_WALL_TEMP"
        """

        self._bh_type = BoreholeType.SINGLE_U_TUBE
        self._boundary_condition = set_boundary_condition_enum(boundary_condition)
        self._bh = SingleUBorehole(
            borehole_diameter,
            pipe_outer_diameter,
            pipe_dimension_ratio,
            length,
            shank_space,
            pipe_conductivity,
            grout_conductivity,
            soil_conductivity,
            fluid_type,
            fluid_concentration,
        )

    def init_double_u_borehole(
        self,
        borehole_diameter: float,
        pipe_outer_diameter: float,
        pipe_dimension_ratio: float,
        length: float,
        shank_space: float,
        pipe_conductivity: float,
        pipe_inlet_arrangement: str,
        grout_conductivity: float,
        soil_conductivity: float,
        fluid_type: str,
        fluid_concentration: float = 0,
        boundary_condition: str = "UNIFORM_HEAT_FLUX",
    ):
        """
        Constructs a grouted double u-tube borehole with u-tubes in parallel.

        :param borehole_diameter: borehole diameter, in m.
        :param pipe_outer_diameter: outer diameter of the pipe, in m.
        :param pipe_dimension_ratio: non-dimensional ratio of pipe diameter to pipe thickness.
        :param length: length of borehole from top to bottom, in m.
        :param shank_space: radial distance from the borehole center to the pipe center, in m.
        :param pipe_conductivity: pipe thermal conductivity, in W/m-K.
        :param pipe_inlet_arrangement: arrangement of the pipe inlets. "ADJACENT", or "DIAGONAL"
        :param grout_conductivity: grout thermal conductivity, in W/m-K.
        :param soil_conductivity: soil thermal conductivity, in W/m-K.
        :param fluid_type: fluid type. "ETHYLALCOHOL", "ETHYLENEGLYCOL", "METHYLALCOHOL",  "PROPYLENEGLYCOL", or "WATER"
        :param fluid_concentration: fractional concentration of antifreeze mixture, from 0-0.6.
        :param boundary_condition: borehole wall boundary condition. "UNIFORM_HEAT_FLUX" or "UNIFORM_BOREHOLE_WALL_TEMP"
        """

        self._bh_type = BoreholeType.DOUBLE_U_TUBE
        self._boundary_condition = set_boundary_condition_enum(boundary_condition)
        self._bh = DoubleUTube(
            borehole_diameter,
            pipe_outer_diameter,
            pipe_dimension_ratio,
            length,
            shank_space,
            pipe_conductivity,
            pipe_inlet_arrangement,
            grout_conductivity,
            soil_conductivity,
            fluid_type,
            fluid_concentration,
        )

    def init_coaxial_borehole(
        self,
        borehole_diameter: float,
        outer_pipe_outer_diameter: float,
        outer_pipe_dimension_ratio: float,
        outer_pipe_conductivity: float,
        inner_pipe_outer_diameter: float,
        inner_pipe_dimension_ratio: float,
        inner_pipe_conductivity: float,
        length: float,
        grout_conductivity: float,
        soil_conductivity: float,
        fluid_type: str,
        fluid_concentration: float,
        boundary_condition: str = "UNIFORM_HEAT_FLUX",
    ):
        """
        Constructs a grouted coaxial borehole.

        :param borehole_diameter: borehole diameter, in m.
        :param outer_pipe_outer_diameter: outer diameter of outer pipe, in m.
        :param outer_pipe_dimension_ratio: non-dimensional ratio of outer pipe diameter to thickness.
        :param outer_pipe_conductivity: outer pipe thermal conductivity, in W/m-K.
        :param inner_pipe_outer_diameter: inner diameter of outer pipe, in m.
        :param inner_pipe_dimension_ratio: non-dimensional ratio of inner pipe diameter to thickness.
        :param inner_pipe_conductivity: inner pipe thermal conductivity, in W/m-K.
        :param length: length of borehole from top to bottom, in m.
        :param grout_conductivity: grout thermal conductivity, in W/m-K.
        :param soil_conductivity: pipe thermal conductivity, in W/m-K.
        :param fluid_type: fluid type. "ETHYLALCOHOL", "ETHYLENEGLYCOL", "METHYLALCOHOL",  "PROPYLENEGLYCOL", or "WATER"
        :param fluid_concentration: fractional concentration of antifreeze mixture, from 0-0.6.
        :param boundary_condition: borehole wall boundary condition. "UNIFORM_HEAT_FLUX" or "UNIFORM_BOREHOLE_WALL_TEMP"
        """

        self._bh_type = BoreholeType.COAXIAL
        self._boundary_condition = set_boundary_condition_enum(boundary_condition)
        self._bh = Coaxial(
            borehole_diameter,
            outer_pipe_outer_diameter,
            outer_pipe_dimension_ratio,
            outer_pipe_conductivity,
            inner_pipe_outer_diameter,
            inner_pipe_dimension_ratio,
            inner_pipe_conductivity,
            length,
            grout_conductivity,
            soil_conductivity,
            fluid_type,
            fluid_concentration,
        )

    def init_from_dict(self, inputs: dict):
        """
        Constructs a borehole from a set of dictionary inputs.

        :param inputs: dict of input data.
        """

        bh_type_str = inputs["borehole_type"].upper()
        if bh_type_str == BoreholeType.SINGLE_U_TUBE.name:
            self._bh_type = BoreholeType.SINGLE_U_TUBE
        elif bh_type_str == BoreholeType.DOUBLE_U_TUBE.name:
            self._bh_type = BoreholeType.DOUBLE_U_TUBE
        elif bh_type_str == BoreholeType.COAXIAL.name:
            self._bh_type = BoreholeType.COAXIAL
        else:
            raise LookupError(f'borehole_type "{bh_type_str}" not supported')

        bc_str = inputs["boundary_condition"].upper()
        if bc_str == BoundaryCondition.UNIFORM_HEAT_FLUX.name:
            self._boundary_condition = BoundaryCondition.UNIFORM_HEAT_FLUX
        elif bc_str == BoundaryCondition.UNIFORM_BOREHOLE_WALL_TEMP.name:
            self._boundary_condition = BoundaryCondition.UNIFORM_BOREHOLE_WALL_TEMP
        else:
            raise LookupError(f'boundary_condition "{bc_str}" not supported')

        bh_diameter = inputs["borehole_diameter"]
        length = inputs["length"]
        grout_conductivity = inputs["grout_conductivity"]
        soil_conductivity = inputs["soil_conductivity"]
        fluid_type = inputs["fluid_type"]
        fluid_concentration = inputs["fluid_concentration"]

        if self._bh_type == BoreholeType.SINGLE_U_TUBE:
            pipe_outer_dia_single = inputs["single_u_tube"]["pipe_outer_diameter"]
            dimension_ratio_single = inputs["single_u_tube"]["pipe_dimension_ratio"]
            shank_space_single = inputs["single_u_tube"]["shank_space"]
            pipe_conductivity_single = inputs["single_u_tube"]["pipe_conductivity"]

            self._bh = SingleUBorehole(
                bh_diameter,
                pipe_outer_dia_single,
                dimension_ratio_single,
                length,
                shank_space_single,
                pipe_conductivity_single,
                grout_conductivity,
                soil_conductivity,
                fluid_type,
                fluid_concentration,
            )

        elif self._bh_type == BoreholeType.DOUBLE_U_TUBE:
            pipe_outer_dia_double = inputs["double_u_tube"]["pipe_outer_diameter"]
            dimension_ratio_double = inputs["double_u_tube"]["pipe_dimension_ratio"]
            shank_space_double = inputs["double_u_tube"]["shank_space"]
            pipe_conductivity_double = inputs["double_u_tube"]["pipe_conductivity"]
            pipe_inlet_arrangement = inputs["double_u_tube"]["pipe_inlet_arrangement"]

            self._bh = DoubleUTube(
                bh_diameter,
                pipe_outer_dia_double,
                dimension_ratio_double,
                length,
                shank_space_double,
                pipe_conductivity_double,
                pipe_inlet_arrangement,
                grout_conductivity,
                soil_conductivity,
                fluid_type,
                fluid_concentration,
            )

        elif self._bh_type == BoreholeType.COAXIAL:
            pipe_outer_dia_coax = inputs["coaxial"]["outer_pipe_outer_diameter"]
            outer_pipe_dimension_ratio = inputs["coaxial"]["outer_pipe_dimension_ratio"]
            pipe_conductivity_coax = inputs["coaxial"]["outer_pipe_conductivity"]
            inner_pipe_outer_diameter = inputs["coaxial"]["inner_pipe_outer_diameter"]
            inner_pipe_dimension_ratio = inputs["coaxial"]["inner_pipe_dimension_ratio"]
            inner_pipe_conductivity = inputs["coaxial"]["inner_pipe_conductivity"]

            self._bh = Coaxial(
                bh_diameter,
                pipe_outer_dia_coax,
                outer_pipe_dimension_ratio,
                pipe_conductivity_coax,
                inner_pipe_outer_diameter,
                inner_pipe_dimension_ratio,
                inner_pipe_conductivity,
                length,
                grout_conductivity,
                soil_conductivity,
                fluid_type,
                fluid_concentration,
            )

        else:
            raise NotImplementedError(f'bh_type "{self._bh_type.name}" not implemented')

    def calc_bh_resist(self, mass_flow_rate, temperature):
        """
        Computes the effective borehole thermal resistance.

        :param mass_flow_rate: total borehole mass flow rate, in kg/s
        :param temperature: average fluid temperature, in Celsius
        :return: effective borehole resistance, in K/W-m
        """

        if self._bh is None:
            raise TypeError("Borehole not initialized")

        if self._boundary_condition == BoundaryCondition.UNIFORM_HEAT_FLUX:
            return self._bh.calc_effective_bh_resistance_uhf(mass_flow_rate, temperature)

        if self._boundary_condition == BoundaryCondition.UNIFORM_BOREHOLE_WALL_TEMP:
            return self._bh.calc_effective_bh_resistance_ubwt(mass_flow_rate, temperature)
